@@ -1,7 +1,7 @@
 /**************************************************************************
 **
-** Copyright (c) 2014 Brian McGillion
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 Brian McGillion
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,20 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -30,8 +31,8 @@
 #include "mercurialclient.h"
 #include "constants.h"
 
-#include <vcsbase/command.h>
-#include <vcsbase/vcsbaseoutputwindow.h>
+#include <vcsbase/vcscommand.h>
+#include <vcsbase/vcsoutputwindow.h>
 #include <vcsbase/vcsbaseplugin.h>
 #include <vcsbase/vcsbaseeditor.h>
 #include <vcsbase/vcsbaseeditorparameterwidget.h>
@@ -46,17 +47,20 @@
 #include <QTextStream>
 #include <QVariant>
 
+using namespace Utils;
+using namespace VcsBase;
+
 namespace Mercurial {
 namespace Internal  {
 
 MercurialClient::MercurialClient(MercurialSettings *settings) :
-    VcsBase::VcsBaseClient(settings)
+    VcsBaseClient(settings)
 {
 }
 
 MercurialSettings *MercurialClient::settings() const
 {
-    return dynamic_cast<MercurialSettings *>(VcsBase::VcsBaseClient::settings());
+    return dynamic_cast<MercurialSettings *>(VcsBaseClient::settings());
 }
 
 bool MercurialClient::manifestSync(const QString &repository, const QString &relativeFilename)
@@ -88,9 +92,9 @@ bool MercurialClient::synchronousClone(const QString &workingDir,
     Q_UNUSED(extraOptions);
     QDir workingDirectory(srcLocation);
     QByteArray output;
-    const unsigned flags = VcsBase::VcsBasePlugin::SshPasswordPrompt |
-            VcsBase::VcsBasePlugin::ShowStdOutInLogWindow |
-            VcsBase::VcsBasePlugin::ShowSuccessMessage;
+    const unsigned flags = VcsBasePlugin::SshPasswordPrompt |
+            VcsBasePlugin::ShowStdOutInLogWindow |
+            VcsBasePlugin::ShowSuccessMessage;
 
     if (workingDirectory.exists()) {
         // Let's make first init
@@ -101,33 +105,33 @@ bool MercurialClient::synchronousClone(const QString &workingDir,
         // Then pull remote repository
         arguments.clear();
         arguments << QLatin1String("pull") << dstLocation;
-        const Utils::SynchronousProcessResponse resp1 =
+        const SynchronousProcessResponse resp1 =
                 vcsSynchronousExec(workingDirectory.path(), arguments, flags);
-        if (resp1.result != Utils::SynchronousProcessResponse::Finished)
+        if (resp1.result != SynchronousProcessResponse::Finished)
             return false;
 
         // By now, there is no hgrc file -> create it
-        Utils::FileSaver saver(workingDirectory.path() + QLatin1String("/.hg/hgrc"));
+        FileSaver saver(workingDirectory.path() + QLatin1String("/.hg/hgrc"));
         const QString hgrc = QLatin1String("[paths]\ndefault = ") + dstLocation + QLatin1Char('\n');
         saver.write(hgrc.toUtf8());
         if (!saver.finalize()) {
-            VcsBase::VcsBaseOutputWindow::instance()->appendError(saver.errorString());
+            VcsOutputWindow::appendError(saver.errorString());
             return false;
         }
 
         // And last update repository
         arguments.clear();
         arguments << QLatin1String("update");
-        const Utils::SynchronousProcessResponse resp2 =
+        const SynchronousProcessResponse resp2 =
                 vcsSynchronousExec(workingDirectory.path(), arguments, flags);
-        return resp2.result == Utils::SynchronousProcessResponse::Finished;
+        return resp2.result == SynchronousProcessResponse::Finished;
     } else {
         QStringList arguments(QLatin1String("clone"));
         arguments << dstLocation << workingDirectory.dirName();
         workingDirectory.cdUp();
-        const Utils::SynchronousProcessResponse resp =
+        const SynchronousProcessResponse resp =
                 vcsSynchronousExec(workingDirectory.path(), arguments, flags);
-        return resp.result == Utils::SynchronousProcessResponse::Finished;
+        return resp.result == SynchronousProcessResponse::Finished;
     }
 }
 
@@ -137,18 +141,18 @@ bool MercurialClient::synchronousPull(const QString &workingDir, const QString &
     args << vcsCommandString(PullCommand) << extraOptions << srcLocation;
     // Disable UNIX terminals to suppress SSH prompting
     const unsigned flags =
-            VcsBase::VcsBasePlugin::SshPasswordPrompt
-            | VcsBase::VcsBasePlugin::ShowStdOutInLogWindow
-            | VcsBase::VcsBasePlugin::ShowSuccessMessage;
-    const QString binary = settings()->binaryPath();
+            VcsBasePlugin::SshPasswordPrompt
+            | VcsBasePlugin::ShowStdOutInLogWindow
+            | VcsBasePlugin::ShowSuccessMessage;
+    const FileName binary = settings()->binaryPath();
     const int timeoutSec = settings()->value(settings()->timeoutKey).toInt();
 
     // cause mercurial doesn`t understand LANG
     QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
     env.insert(QLatin1String("LANGUAGE"), QLatin1String("C"));
-    const Utils::SynchronousProcessResponse resp = VcsBase::VcsBasePlugin::runVcs(
+    const SynchronousProcessResponse resp = VcsBasePlugin::runVcs(
                 workingDir, binary, args, timeoutSec * 1000, flags, 0, env);
-    const bool ok = resp.result == Utils::SynchronousProcessResponse::Finished;
+    const bool ok = resp.result == SynchronousProcessResponse::Finished;
 
     parsePullOutput(resp.stdOut.trimmed());
     return ok;
@@ -190,22 +194,21 @@ QStringList MercurialClient::parentRevisionsSync(const QString &workingDirectory
     QByteArray outputData;
     if (!vcsFullySynchronousExec(workingDirectory, args, &outputData))
         return QStringList();
-    const QString output = Utils::SynchronousProcess::normalizeNewlines(
+    const QString output = SynchronousProcess::normalizeNewlines(
                 QString::fromLocal8Bit(outputData));
     /* Looks like: \code
 changeset:   0:031a48610fba
 user: ...
 \endcode   */
     // Obtain first line and split by blank-delimited tokens
-    VcsBase::VcsBaseOutputWindow *outputWindow = VcsBase::VcsBaseOutputWindow::instance();
     const QStringList lines = output.split(QLatin1Char('\n'));
     if (lines.size() < 1) {
-        outputWindow->appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(output)));
+        VcsOutputWindow::appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(output)));
         return QStringList();
     }
     QStringList changeSets = lines.front().simplified().split(QLatin1Char(' '));
     if (changeSets.size() < 2) {
-        outputWindow->appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(output)));
+        VcsOutputWindow::appendSilently(msgParentRevisionFailed(workingDirectory, revision, msgParseParentsOutputFailed(output)));
         return QStringList();
     }
     // Remove revision numbers
@@ -284,9 +287,9 @@ void MercurialClient::incoming(const QString &repositoryRoot, const QString &rep
 
     const QString title = tr("Hg incoming %1").arg(id);
 
-    VcsBase::VcsBaseEditorWidget *editor = createVcsEditor(Constants::DIFFLOG, title, repositoryRoot,
-                                                     true, "incoming", id);
-    VcsBase::Command *cmd = createCommand(repository, editor);
+    VcsBaseEditorWidget *editor = createVcsEditor(Constants::DIFFLOG_ID, title, repositoryRoot,
+                                                  true, "incoming", id);
+    VcsCommand *cmd = createCommand(repository, editor);
     enqueueJob(cmd, args);
 }
 
@@ -298,15 +301,15 @@ void MercurialClient::outgoing(const QString &repositoryRoot)
     const QString title = tr("Hg outgoing %1").
             arg(QDir::toNativeSeparators(repositoryRoot));
 
-    VcsBase::VcsBaseEditorWidget *editor = createVcsEditor(Constants::DIFFLOG, title, repositoryRoot, true,
-                                                     "outgoing", repositoryRoot);
+    VcsBaseEditorWidget *editor = createVcsEditor(Constants::DIFFLOG_ID, title, repositoryRoot, true,
+                                                  "outgoing", repositoryRoot);
 
-    VcsBase::Command *cmd = createCommand(repositoryRoot, editor);
+    VcsCommand *cmd = createCommand(repositoryRoot, editor);
     enqueueJob(cmd, args);
 }
 
 void MercurialClient::annotate(const QString &workingDir, const QString &file,
-                               const QString revision, int lineNumber,
+                               const QString &revision, int lineNumber,
                                const QStringList &extraOptions)
 {
     QStringList args(extraOptions);
@@ -349,7 +352,8 @@ void MercurialClient::view(const QString &source, const QString &id,
                            const QStringList &extraOptions)
 {
     QStringList args;
-    args << QLatin1String("log") << QLatin1String("-p") << QLatin1String("-g");
+    args << QLatin1String("-v") << QLatin1String("log")
+         << QLatin1String("-p") << QLatin1String("-g");
     VcsBaseClient::view(source, id, args << extraOptions);
 }
 
@@ -357,19 +361,19 @@ QString MercurialClient::findTopLevelForFile(const QFileInfo &file) const
 {
     const QString repositoryCheckFile = QLatin1String(Constants::MERCURIALREPO) + QLatin1String("/requires");
     return file.isDir() ?
-                VcsBase::VcsBasePlugin::findRepositoryForDirectory(file.absoluteFilePath(), repositoryCheckFile) :
-                VcsBase::VcsBasePlugin::findRepositoryForDirectory(file.absolutePath(), repositoryCheckFile);
+                VcsBasePlugin::findRepositoryForDirectory(file.absoluteFilePath(), repositoryCheckFile) :
+                VcsBasePlugin::findRepositoryForDirectory(file.absolutePath(), repositoryCheckFile);
 }
 
-Core::Id MercurialClient::vcsEditorKind(VcsCommand cmd) const
+Core::Id MercurialClient::vcsEditorKind(VcsCommandTag cmd) const
 {
     switch (cmd) {
     case AnnotateCommand:
-        return Constants::ANNOTATELOG;
+        return Constants::ANNOTATELOG_ID;
     case DiffCommand:
-        return Constants::DIFFLOG;
+        return Constants::DIFFLOG_ID;
     case LogCommand:
-        return Constants::FILELOG;
+        return Constants::FILELOG_ID;
     default:
         return Core::Id();
     }
@@ -432,13 +436,13 @@ struct MercurialDiffParameters
 };
 
 // Parameter widget controlling whitespace diff mode, associated with a parameter
-class MercurialDiffParameterWidget : public VcsBase::VcsBaseEditorParameterWidget
+class MercurialDiffParameterWidget : public VcsBaseEditorParameterWidget
 {
     Q_OBJECT
 public:
     MercurialDiffParameterWidget(MercurialClient *client,
                                  const MercurialDiffParameters &p, QWidget *parent = 0) :
-        VcsBase::VcsBaseEditorParameterWidget(parent), m_client(client), m_params(p)
+        VcsBaseEditorParameterWidget(parent), m_client(client), m_params(p)
     {
         mapSetting(addToggleButton(QLatin1String("-w"), tr("Ignore Whitespace")),
                    client->settings()->boolPointer(MercurialSettings::diffIgnoreWhiteSpaceKey));
@@ -456,7 +460,7 @@ private:
     const MercurialDiffParameters m_params;
 };
 
-VcsBase::VcsBaseEditorParameterWidget *MercurialClient::createDiffEditor(
+VcsBaseEditorParameterWidget *MercurialClient::createDiffEditor(
     const QString &workingDir, const QStringList &files, const QStringList &extraOptions)
 {
     MercurialDiffParameters parameters;
