@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -40,6 +40,7 @@
 
 #include <utils/detailswidget.h>
 #include <utils/qtcassert.h>
+#include <utils/algorithm.h>
 
 #include <QAction>
 #include <QApplication>
@@ -246,20 +247,18 @@ ToolChainConfigWidget *ToolChainModel::widget(const QModelIndex &index)
 
 bool ToolChainModel::isDirty() const
 {
-    foreach (ToolChainNode *n, m_manualRoot->childNodes) {
-        if (n->changed)
-            return true;
-    }
-    return false;
+    return Utils::anyOf(m_manualRoot->childNodes,
+                        [](ToolChainNode *n) {
+                            return n->changed;
+                        });
 }
 
 bool ToolChainModel::isDirty(ToolChain *tc) const
 {
-    foreach (ToolChainNode *n, m_manualRoot->childNodes) {
-        if (n->toolChain == tc && n->changed)
-            return true;
-    }
-    return false;
+    return Utils::anyOf(m_manualRoot->childNodes,
+                        [tc](ToolChainNode *n) {
+                            return n->toolChain == tc && n->changed;
+                        });
 }
 
 void ToolChainModel::setDirty()
@@ -329,13 +328,7 @@ void ToolChainModel::apply()
 
 void ToolChainModel::markForRemoval(ToolChain *tc)
 {
-    ToolChainNode *node = 0;
-    foreach (ToolChainNode *n, m_manualRoot->childNodes) {
-        if (n->toolChain == tc) {
-            node = n;
-            break;
-        }
-    }
+    ToolChainNode *node = findToolChain(m_manualRoot->childNodes, tc);
     if (node) {
         emit beginRemoveRows(index(m_manualRoot), m_manualRoot->childNodes.indexOf(node), m_manualRoot->childNodes.indexOf(node));
         m_manualRoot->childNodes.removeOne(node);
@@ -381,17 +374,23 @@ ToolChainNode *ToolChainModel::createNode(ToolChainNode *parent, ToolChain *tc,
     return node;
 }
 
+ToolChainNode *ToolChainModel::findToolChain(const QList<ToolChainNode *> &container, ToolChain *tc)
+{
+    return Utils::findOrDefault(container, [tc](ToolChainNode *n) {
+        return n->toolChain == tc;
+    });
+}
+
 void ToolChainModel::addToolChain(ToolChain *tc)
 {
-    QList<ToolChainNode *> nodes = m_toAddList;
-    foreach (ToolChainNode *n, nodes) {
-        if (n->toolChain == tc) {
-            m_toAddList.removeOne(n);
-            // do not delete n: Still used elsewhere!
-            return;
-        }
+    ToolChainNode *n = findToolChain(m_toAddList, tc);
+    if (n) {
+        m_toAddList.removeOne(n);
+        // do not delete n: Still used elsewhere!
+        return;
     }
 
+
     ToolChainNode *parent = m_manualRoot;
     if (tc->isAutoDetected())
         parent = m_autoRoot;
@@ -406,13 +405,11 @@ void ToolChainModel::addToolChain(ToolChain *tc)
 
 void ToolChainModel::removeToolChain(ToolChain *tc)
 {
-    QList<ToolChainNode *> nodes = m_toRemoveList;
-    foreach (ToolChainNode *n, nodes) {
-        if (n->toolChain == tc) {
-            m_toRemoveList.removeOne(n);
-            delete n;
-            return;
-        }
+    ToolChainNode *n = findToolChain(m_toRemoveList, tc);
+    if (n) {
+        m_toRemoveList.removeOne(n);
+        delete n;
+        return;
     }
 
     ToolChainNode *parent = m_manualRoot;
@@ -491,8 +488,8 @@ QWidget *ToolChainOptionsPage::widget()
         connect(m_model, SIGNAL(toolChainStateChanged()), this, SLOT(updateState()));
 
         m_toolChainView->setModel(m_model);
-        m_toolChainView->header()->setResizeMode(0, QHeaderView::ResizeToContents);
-        m_toolChainView->header()->setResizeMode(1, QHeaderView::Stretch);
+        m_toolChainView->header()->setSectionResizeMode(0, QHeaderView::ResizeToContents);
+        m_toolChainView->header()->setSectionResizeMode(1, QHeaderView::Stretch);
         m_toolChainView->expandAll();
 
         m_selectionModel = m_toolChainView->selectionModel();
