@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -51,6 +51,8 @@
 #include <qtsupport/qtkitinformation.h>
 #include <utils/hostosinfo.h>
 
+#include "api/runenvironment.h"
+
 #include <QFileInfo>
 #include <QFormLayout>
 #include <QLabel>
@@ -62,53 +64,73 @@
 
 using namespace ProjectExplorer;
 
-namespace {
+namespace QbsProjectManager {
+namespace Internal {
 
 const char QBS_RC_PREFIX[] = "Qbs.RunConfiguration:";
 const char COMMAND_LINE_ARGUMENTS_KEY[] = "Qbs.RunConfiguration.CommandLineArguments";
 const char USE_TERMINAL_KEY[] = "Qbs.RunConfiguration.UseTerminal";
 const char USER_WORKING_DIRECTORY_KEY[] = "Qbs.RunConfiguration.UserWorkingDirectory";
 
-QString productFromId(Core::Id id)
+static QString rcNameSeparator() { return QLatin1String("---Qbs.RC.NameSeparator---"); }
+
+static Core::Id idFromProduct(const QbsProject *project, const qbs::ProductData &product)
 {
-    return id.suffixAfter(QBS_RC_PREFIX);
+    QString id = QLatin1String(QBS_RC_PREFIX);
+    id.append(QbsProject::uniqueProductName(product)).append(rcNameSeparator())
+            .append(QbsProject::productDisplayName(project->qbsProject(), product));
+    return Core::Id::fromString(id);
 }
 
-const qbs::ProductData findProduct(const qbs::ProjectData &pro, const QString &name)
+static QString uniqueProductNameFromId(Core::Id id)
+{
+    const QString suffix = id.suffixAfter(QBS_RC_PREFIX);
+    return suffix.left(suffix.indexOf(rcNameSeparator()));
+}
+
+static QString productDisplayNameFromId(Core::Id id)
+{
+    const QString suffix = id.suffixAfter(QBS_RC_PREFIX);
+    const int sepPos = suffix.indexOf(rcNameSeparator());
+    if (sepPos == -1)
+        return suffix;
+    return suffix.mid(sepPos + rcNameSeparator().count());
+}
+
+const qbs::ProductData findProduct(const qbs::ProjectData &pro, const QString &uniqeName)
 {
     foreach (const qbs::ProductData &product, pro.allProducts()) {
-        if (product.name() == name)
+        if (QbsProject::uniqueProductName(product) == uniqeName)
             return product;
     }
     return qbs::ProductData();
 }
 
-} // namespace
-
-namespace QbsProjectManager {
-namespace Internal {
-
 // --------------------------------------------------------------------
 // QbsRunConfiguration:
 // --------------------------------------------------------------------
 
 QbsRunConfiguration::QbsRunConfiguration(Target *parent, Core::Id id) :
     LocalApplicationRunConfiguration(parent, id),
-    m_qbsProduct(productFromId(id)),
+    m_uniqueProductName(uniqueProductNameFromId(id)),
     m_runMode(ApplicationLauncher::Gui),
     m_currentInstallStep(0),
     m_currentBuildStepList(0)
 {
     addExtraAspect(new LocalEnvironmentAspect(this));
 
+    m_runModeForced = false;
+    m_runMode = isConsoleApplication() ? ApplicationLauncher::Console : ApplicationLauncher::Gui;
+
     ctor();
 }
 
 QbsRunConfiguration::QbsRunConfiguration(Target *parent, QbsRunConfiguration *source) :
     LocalApplicationRunConfiguration(parent, source),
-    m_qbsProduct(source->m_qbsProduct),
+    m_uniqueProductName(source->m_uniqueProductName),
     m_commandLineArguments(source->m_commandLineArguments),
     m_runMode(source->m_runMode),
+    m_runModeForced(source->m_runModeForced),
     m_userWorkingDirectory(source->m_userWorkingDirectory),
     m_currentInstallStep(0), // no need to copy this, we will get if from the DC anyway.
     m_currentBuildStepList(0) // ditto
@@ -138,15 +160,17 @@ void QbsRunConfiguration::ctor()
     setDefaultDisplayName(defaultDisplayName());
 
     QbsProject *project = static_cast<QbsProject *>(target()->project());
-    connect(project, SIGNAL(projectParsingStarted()), this, SIGNAL(enabledChanged()));
-    connect(project, SIGNAL(projectParsingDone(bool)), this, SIGNAL(enabledChanged()));
+    connect(project, &QbsProject::projectParsingStarted, this, &RunConfiguration::enabledChanged);
+    connect(project, &QbsProject::projectParsingDone, this, [this](bool success) {
+        if (success && !m_runModeForced)
+            m_runMode = isConsoleApplication() ? ApplicationLauncher::Console
+                                               : ApplicationLauncher::Gui;
+        emit enabledChanged();
+    });
 
-    connect(target(), SIGNAL(activeDeployConfigurationChanged(ProjectExplorer::DeployConfiguration*)),
-            this, SLOT(installStepChanged()));
+    connect(target(), &Target::activeDeployConfigurationChanged,
+            this, &QbsRunConfiguration::installStepChanged);
     installStepChanged();
-
-    if (isConsoleApplication())
-        m_runMode = ApplicationLauncher::Console;
 }
 
 QWidget *QbsRunConfiguration::createConfigurationWidget()
@@ -158,7 +182,8 @@ QVariantMap QbsRunConfiguration::toMap() const
 {
     QVariantMap map(LocalApplicationRunConfiguration::toMap());
     map.insert(QLatin1String(COMMAND_LINE_ARGUMENTS_KEY), m_commandLineArguments);
-    map.insert(QLatin1String(USE_TERMINAL_KEY), m_runMode == ApplicationLauncher::Console);
+    if (m_runModeForced)
+        map.insert(QLatin1String(USE_TERMINAL_KEY), m_runMode == ApplicationLauncher::Console);
     map.insert(QLatin1String(USER_WORKING_DIRECTORY_KEY), m_userWorkingDirectory);
     return map;
 }
@@ -166,8 +191,11 @@ QVariantMap QbsRunConfiguration::toMap() const
 bool QbsRunConfiguration::fromMap(const QVariantMap &map)
 {
     m_commandLineArguments = map.value(QLatin1String(COMMAND_LINE_ARGUMENTS_KEY)).toString();
-    m_runMode = map.value(QLatin1String(USE_TERMINAL_KEY), false).toBool() ?
-                ApplicationLauncher::Console : ApplicationLauncher::Gui;
+    if (map.contains(QLatin1String(USE_TERMINAL_KEY))) {
+        m_runMode = map.value(QLatin1String(USE_TERMINAL_KEY), false).toBool() ?
+                    ApplicationLauncher::Console : ApplicationLauncher::Gui;
+        m_runModeForced = true;
+    }
 
     m_userWorkingDirectory = map.value(QLatin1String(USER_WORKING_DIRECTORY_KEY)).toString();
 
@@ -216,7 +244,7 @@ void QbsRunConfiguration::installStepToBeRemoved(int pos)
 QString QbsRunConfiguration::executable() const
 {
     QbsProject *pro = static_cast<QbsProject *>(target()->project());
-    const qbs::ProductData product = findProduct(pro->qbsProjectData(), m_qbsProduct);
+    const qbs::ProductData product = findProduct(pro->qbsProjectData(), m_uniqueProductName);
 
     if (!product.isValid() || !pro->qbsProject().isValid())
         return QString();
@@ -232,21 +260,16 @@ ApplicationLauncher::Mode QbsRunConfiguration::runMode() const
 bool QbsRunConfiguration::isConsoleApplication() const
 {
     QbsProject *pro = static_cast<QbsProject *>(target()->project());
-    const qbs::ProductData product = findProduct(pro->qbsProjectData(), m_qbsProduct);
-    foreach (const qbs::TargetArtifact &ta, product.targetArtifacts()) {
-        if (ta.isExecutable())
-            return !ta.properties().getProperty(QLatin1String("consoleApplication")).toBool();
-    }
-
-    return false;
+    const qbs::ProductData product = findProduct(pro->qbsProjectData(), m_uniqueProductName);
+    return product.properties().value(QLatin1String("consoleApplication"), false).toBool();
 }
 
 QString QbsRunConfiguration::workingDirectory() const
 {
     EnvironmentAspect *aspect = extraAspect<EnvironmentAspect>();
-    QTC_ASSERT(aspect, baseWorkingDirectory());
+    QTC_ASSERT(aspect, return baseWorkingDirectory());
     return QDir::cleanPath(aspect->environment().expandVariables(
-                Utils::expandMacros(baseWorkingDirectory(), macroExpander())));
+                macroExpander()->expand(baseWorkingDirectory())));
 }
 
 QString QbsRunConfiguration::baseWorkingDirectory() const
@@ -264,7 +287,7 @@ QString QbsRunConfiguration::baseWorkingDirectory() const
 
 QString QbsRunConfiguration::commandLineArguments() const
 {
-    return Utils::QtcProcess::expandMacros(m_commandLineArguments, macroExpander());
+    return macroExpander()->expandProcessArgs(m_commandLineArguments);
 }
 
 QString QbsRunConfiguration::rawCommandLineArguments() const
@@ -291,30 +314,48 @@ void QbsRunConfiguration::setCommandLineArguments(const QString &argumentsString
 
 void QbsRunConfiguration::setRunMode(ApplicationLauncher::Mode runMode)
 {
+    if (m_runMode == runMode)
+        return;
+
+    m_runModeForced = true;
     m_runMode = runMode;
     emit runModeChanged(runMode);
 }
 
 void QbsRunConfiguration::addToBaseEnvironment(Utils::Environment &env) const
 {
-    // TODO: Use environment from Qbs!
+    QbsProject *project = static_cast<QbsProject *>(target()->project());
+    if (project) {
+        const qbs::ProductData product = findProduct(project->qbsProjectData(), m_uniqueProductName);
+        if (product.isValid()) {
+            QProcessEnvironment procEnv = env.toProcessEnvironment();
+            procEnv.insert(QLatin1String("QBS_INSTALL_ROOT"), installRoot());
+            procEnv.insert(QLatin1String("QBS_RUN_FILE_PATH"), executable());
+            qbs::RunEnvironment qbsRunEnv = project->qbsProject().getRunEnvironment(product, installOptions(),
+                    procEnv, QbsManager::settings());
+            procEnv = qbsRunEnv.runEnvironment();
+            if (!procEnv.isEmpty()) {
+                env = Utils::Environment();
+                foreach (const QString &key, procEnv.keys())
+                    env.set(key, procEnv.value(key));
+            }
+        }
+    }
 
     QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(target()->kit());
     if (qtVersion)
         env.prependOrSetLibrarySearchPath(qtVersion->qmakeProperty("QT_INSTALL_LIBS"));
 }
 
-QString QbsRunConfiguration::qbsProduct() const
+QString QbsRunConfiguration::uniqueProductName() const
 {
-    return m_qbsProduct;
+    return m_uniqueProductName;
 }
 
 QString QbsRunConfiguration::defaultDisplayName()
 {
-    QString defaultName;
-    if (!m_qbsProduct.isEmpty())
-        defaultName = m_qbsProduct;
-    else
+    QString defaultName = productDisplayNameFromId(id());
+    if (defaultName.isEmpty())
         defaultName = tr("Qbs Run Configuration");
     return defaultName;
 }
@@ -531,7 +572,7 @@ bool QbsRunConfigurationFactory::canCreate(Target *parent, Core::Id id) const
         return false;
 
     QbsProject *project = static_cast<QbsProject *>(parent->project());
-    return findProduct(project->qbsProjectData(), productFromId(id)).isValid();
+    return findProduct(project->qbsProjectData(), uniqueProductNameFromId(id)).isValid();
 }
 
 RunConfiguration *QbsRunConfigurationFactory::doCreate(Target *parent, Core::Id id)
@@ -577,7 +618,7 @@ QList<Core::Id> QbsRunConfigurationFactory::availableCreationIds(Target *parent,
 
     foreach (const qbs::ProductData &product, project->qbsProjectData().allProducts()) {
         if (product.isRunnable() && product.isEnabled())
-            result << Core::Id::fromString(QString::fromLatin1(QBS_RC_PREFIX) + product.name());
+            result << idFromProduct(project, product);
     }
 
     return result;
@@ -585,7 +626,7 @@ QList<Core::Id> QbsRunConfigurationFactory::availableCreationIds(Target *parent,
 
 QString QbsRunConfigurationFactory::displayNameForId(Core::Id id) const
 {
-    return productFromId(id);
+    return productDisplayNameFromId(id);
 }
 
 bool QbsRunConfigurationFactory::canHandle(Target *t) const
